package com.github.dawnwords.jacoco.badge;

import static com.github.dawnwords.jacoco.badge.JacocoBadgePercentageResult.Type.BRANCH;
import static com.github.dawnwords.jacoco.badge.JacocoBadgePercentageResult.Type.CLASS;
import static com.github.dawnwords.jacoco.badge.JacocoBadgePercentageResult.Type.INSTRUCTION;
import static com.github.dawnwords.jacoco.badge.JacocoBadgePercentageResult.Type.LINE;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

import com.github.dawnwords.jacoco.badge.JacocoBadgePercentageResult.Type;
import org.gradle.api.GradleException;
import org.gradle.internal.impldep.com.google.common.collect.ImmutableMap;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

public class JacocoBadgePercentageResultTest {

  @Mock
  private NamedNodeMap attr;

  @Mock
  private Node type;
  @Mock
  private Node covered;
  @Mock
  private Node missed;

  @BeforeMethod
  public void initMocks() {
    MockitoAnnotations.initMocks(this);
    when(attr.getNamedItem("type")).thenReturn(type);
    when(attr.getNamedItem("covered")).thenReturn(covered);
    when(attr.getNamedItem("missed")).thenReturn(missed);
  }

  @DataProvider
  public Object[][] test() {
    return new Object[][]{
        new Object[]{INSTRUCTION, "100", "0", "INSTRUCTION:100.00%",
            "https://img.shields.io/badge/instruction--coverage-100.00%25-brightgreen.svg"},
        new Object[]{BRANCH, "70", "30", "BRANCH:70.00%",
            "https://img.shields.io/badge/branch--coverage-70.00%25-yellow.svg"},
        new Object[]{CLASS, "50", "50", "CLASS:50.00%",
            "https://img.shields.io/badge/class--coverage-50.00%25-orange.svg"},
        new Object[]{LINE, "30", "70", "LINE:30.00%",
            "https://img.shields.io/badge/line--coverage-30.00%25-red.svg"},
    };
  }

  @Test(dataProvider = "test")
  public void test(Type resultType, String coveredValue, String missedValue,
                   String expectedString, String expectedBadge) {
    when(type.getNodeValue()).thenReturn(resultType.name());
    when(covered.getNodeValue()).thenReturn(coveredValue);
    when(missed.getNodeValue()).thenReturn(missedValue);

    JacocoBadgePercentageResult result = new JacocoBadgePercentageResult(attr);
    result.verifyLimit();
    assertEquals(result.toString(), expectedString);
    assertEquals(result.badgeUrl(), expectedBadge);
  }

  @Test(expectedExceptions = GradleException.class, expectedExceptionsMessageRegExp =
      "BRANCH coverage limit not satisfied, expect at least 90.10%, got 70.00%")
  public void testLimitViolation() {
    when(type.getNodeValue()).thenReturn(BRANCH.name());
    when(covered.getNodeValue()).thenReturn("70");
    when(missed.getNodeValue()).thenReturn("30");
    new JacocoBadgePercentageResult(attr, ImmutableMap.of(BRANCH.name(), 90.1)).verifyLimit();
  }
}
